require 'ostruct'
require 'pathname'
require 'thor'
require 'forwardable'

require 'flux/util'

module Flux
  NAME     = '.flux'
  RC       = "#{NAME}.rb"
  RC_LOCAL = "#{NAME}.local.rb"

  class FluxError < StandardError; end
  class TrackerError < FluxError; end

  class << self
    def environment
      @project ||= {}
    end

    def setup
      load rc
      load rc_local if File.exist?(rc_local)

      require 'flux/cli'
    end

    def rc
      find_upwards(RC, Dir.pwd) or
        raise FluxError, "Could not find a '#{RC}' " <<
                         "file in the current filesystem hierarchy."
    end

    def rc_local
      find_upwards(RC_LOCAL, Dir.pwd)
    end

    def find_upwards(object, start_dir)
      p  = Pathname(start_dir)
      f  = (p + object).expand_path.to_s

      if File.exist?(f)
        f
      elsif p == p.parent
        nil
      else
        find_upwards(object, p.parent)
      end
    end
  end
end

module Kernel
  def use(mod, options = {})
    (Flux.environment[mod.to_s] ||= {}).
      merge! Hash[*options.map { |k, v| [k.to_s, v] }.flatten]


  end
end
