require 'flux/git'

module Flux
  module CLI
    class Feature < Thor
      B = Flux::Git::Branch

      namespace :feature

      desc "start STORY_ID BRANCH", "start working on a story"
      method_option :estimate,      :type    => :numeric
      method_option :parent_branch, :type    => :string,
                                    :default => 'master',
                                    :aliases => '-b'
      def start(story_id, branch_id = branch_id_from_story(story_id))
        invoke 'pt:grab', [story_id], :estimate => options[:estimate]
        invoke 'pt:start', [story_id]
        invoke 'branch:create',
               [branch_id],
               :parent_branch => options[:parent_branch]

        link story_id, branch_id
      end

      desc "link STORY_ID BRANCH", "link a story to a branch"
      def link(story_id, branch_id)
        B.local(branch_id).config('story_id', story_id)
      end

      desc "finish", "finish the current feature"
      def finish(branch_id = B.current.name)
        invoke "pt:finish", [B.local(branch_id).config('story_id')]
        invoke "review:request"
      end

      private

      def branch_id_from_story(story_id)
        project  = Flux::PT::Project.current
        story    = project.stories.find(story_id)
        words    = story.
          name.
          strip.
          downcase.
          split(/\s+/).
          select { |w| w.size > 2 }.
          last(7)
        initials = project.members.me.initials.downcase

        "#{initials}/#{story.id}-#{words.join('_')}"
      end
    end
  end
end
