require 'flux/git'

module Flux
  module CLI
    class Git < Thor
      G = Flux::Git

      namespace :git

      desc "fetch", "fetch changes from remote repos"
      def fetch
        G.git.fetch
      end
    end

    class Branch < Thor
      B = Flux::Git::Branch

      namespace :branch

      desc "create BRANCH", "create a branch as a public, tracked branch"
      method_option :parent_branch, :type    => :string,
                                    :default => 'master',
                                    :aliases => '-p'
      def create(branch_id, options = {})
        up     = B.remote(branch_id, 'origin')
        parent = B.local(options[:parent_branch])
        branch = B.local(branch_id).
          create(parent).
          publish(up).
          track(up).
          checkout
      end

      class Checks < Thor
        G = Flux::Git
        B = G::Branch

        namespace 'branch:checks'

        desc "synced [BRANCH]", "check if branch is synced with upstream"
        def synced(branch = B.current)
          branch = G::Branch(branch)

          unless branch.synced?
            raise Error, "Local branch `#{branch.name}' is not synced."
          end
        end

        desc "rebased [BRANCH]", "check if branch is rebased with other branch"
        method_option :parent, :type    => :string,
                               :default => 'master',
                               :aliases => '-p'
        def rebased(branch = B.current)
          branch = G::Branch(branch)
          parent = G::Branch(options[:parent])

          unless branch.includes?(parent)
            raise Error,
                  "`#{branch.fqdn}' is not rebased onto `#{parent.fqdn}'."
          end
        end
      end
    end
  end
end
