require 'pivotal-tracker'
require 'flux/pivotal_tracker'
require 'flux/ext/pivotal-tracker'

module Flux
  module CLI
    class PT < Thor
      STORY_LIST_HEADERS = %w(>ID STATE ASSIGNEE >EST STORY)

      extend  Forwardable
      include Flux::Util

      namespace :pt

      default_task :list

      class << self
        def login
          ::PivotalTracker::Client.token = Flux::PT.config['token']
        end

        def story_update(name, attrs = nil, &get_attrs)
          desc "#{name} STORY_ID", "#{name} a story"
          method_option :estimate, :type => :numeric, :aliases => '-e'
          define_method name do |story_id|
            a = attrs || instance_eval(&get_attrs)

            a[:estimate] = options[:estimate] if options[:estimate]

            story(story_id).update a
          end
        end
      end

      no_tasks {
        def_delegator self, :config
      }

      login if Flux::PT.config

      desc "list", "list my stories, or all, excluding icebox"
      method_option :owner, :type    => :string,
                            :default => 'me',
                            :aliases => '-o'
      def list
        stories = begin
                    if %w(none n).include?(options[:owner])
                      current_project.stories.unassigned
                    elsif %w(all a).include?(options[:owner])
                      current_project.stories.scheduled
                    else
                      m = if options[:owner] == 'me'
                            current_project.members.me
                          else
                            current_project.members.find { |e|
                              e.name == options[:member]
                            }
                          end

                      current_project.stories.owned_by(m)
                    end
                  end

        stories.reject! { |s| s.state == 'accepted' }

        table   =
          [STORY_LIST_HEADERS] +
          stories.map { |s|
            [s.id, s.state, s.owned_by, s.est, s.name]
          }

        puts_table table
      end

      story_update :finish, :current_state => 'finished'
      story_update(:grab)   {{:owned_by => current_project.members.me.name}}
      story_update :start,  :current_state => 'started'

      private

      def current_project
        Flux::PT::Project.current
      end

      def story(id)
        current_project.stories.find(id)
      end
    end
  end
end
